@@ -28,11 +28,15 @@ If you make modifications while inside a context (using the Python context
 manager protocol), the modifications are only applied when the context
 exits for the last time.
 
+For tokenized documents (see :class:`parce.Document`), parce inherits from this
+base class (see the :mod:`~parce.treedocument` module).
+
 You can use a Cursor to keep track of positions in a document. The position
 (and selection) of a Cursor is adjusted when the text in the document is
 changed.
 
-For tokenized documents, parce inherits from this base class.
+You can use the various ``find_block()`` and ``blocks()`` methods to iterate
+over a Document on a line-by-line basis.
 
 """
 
@@ -65,8 +69,8 @@ class AbstractDocument:
     last exits.
     To make a Document work, you should at least implement:
 
-     *   ``text()``
-     *   ``_update_contents()``
+    * ``text()``
+    * ``_update_contents()``
 
     The method text() should simply return the entire text string.
 
@@ -77,15 +81,19 @@ class AbstractDocument:
 
     For efficiency reasons, you might want to reimplement:
 
-     *  ``set_text()``
-     *  ``__len__()``
-     *  ``_get_contents()`` (called by ``__getitem__``)
+    * ``set_text()``
+    * ``__len__()``
+    * ``_get_contents()`` (called by ``__getitem__``)
 
     """
+
+    block_separator = '\n'  #: separator to use for block boundaries (newline)
+
     def __init__(self):
         super().__init__()
         self._cursors = weakref.WeakSet()
         self._edit_context = 0
+        self._revision = 0
         self._changes = []
 
     def text(self):
@@ -131,6 +139,7 @@ class AbstractDocument:
             self._apply_changes()
 
     def __setitem__(self, key, text):
+        """Replace the position or slice with text."""
         start, end = self._parse_key(key)
         if ((text or start != end) and
             (end - start != len(text) or self[start:end] != text)):
@@ -139,7 +148,7 @@ class AbstractDocument:
                 self._apply_changes()
 
     def __delitem__(self, key):
-        """Delete the slice of text."""
+        """Delete the chracter or slice of text."""
         self[key] = ""
 
     def __getitem__(self, key):
@@ -156,8 +165,8 @@ class AbstractDocument:
         total = len(self)
         if isinstance(key, slice):
             start, end, _ = key.indices(total)
-        elif isinstance(key, Cursor):
-            start, end, _ = slice(key.start, key.end).indices(total)
+        elif isinstance(key, AbstractTextRange):
+            start, end, _ = slice(key.pos, key.end).indices(total)
         else:
             # single integer
             if key < 0:
@@ -192,20 +201,21 @@ class AbstractDocument:
             self._update_cursors()
             self._update_contents()
             self._changes.clear()
+            self._revision += 1
             self.contents_changed(head, end - head, added)
 
     def _update_cursors(self):
         """Update the positions of the cursors."""
         i = 0
-        cursors = sorted(self._cursors, key = lambda c: c.start)
+        cursors = sorted(self._cursors, key = lambda c: c.pos)
         for start, end, text in self._changes:
             for c in cursors[i:]:
-                ahead = c.start > start
+                ahead = c.pos > start
                 if ahead:
-                    if end >= c.start:
-                        c.start = start
+                    if end >= c.pos:
+                        c.pos = start
                     else:
-                        c.start += start + len(text) - end
+                        c.pos += start + len(text) - end
                 if c.end is not None and c.end >= start:
                     if end >= c.end:
                         c.end = start + len(text)
@@ -218,11 +228,59 @@ class AbstractDocument:
         """Should apply the changes (in self._changes) to the text."""
         raise NotImplementedError
 
+    def revision(self):
+        """Return the revision number.
+
+        This number is incremented by one on every document change.
+
+        """
+        return self._revision
+
+    def append(self, text):
+        """Append text at the end of the document."""
+        self.insert(len(self), text)
+
     def insert(self, pos, text):
         """Insert text at pos."""
         if text:
             self[pos:pos] = text
 
+    def find_start_of_block(self, position):
+        """Find the start of the block the position is in."""
+        sep = self.block_separator
+        return self[:position].rfind(sep) + len(sep)
+
+    def find_end_of_block(self, position):
+        """Find the end of the block the position is in."""
+        pos = self[position:].find(self.block_separator)
+        if pos == -1:
+            return len(self)
+        return position + pos
+
+    def find_block(self, position):
+        """Return a Block representing the text line (block) at position."""
+        pos = self.find_start_of_block(position)
+        end = self.find_end_of_block(position)
+        return Block(self, pos, end)
+
+    def blocks(self, start=0, end=None):
+        """Yield Blocks, starting at position start, ending at end.
+
+        Start defaults to 0, end to None, which means iterate to the last block.
+
+        """
+        block = self.find_block(start)
+        if end is None:
+            while block:
+                yield block
+                block = block.next_block()
+        else:
+            while True:
+                yield block
+                block = block.next_block()
+                if not block or block.pos >= end:
+                    break
+
     def replace(self, old, new, start=0, end=None, count=0):
         """Replace occurrences of old with new in region start->end.
 
@@ -313,6 +371,9 @@ class Document(AbstractDocument, util.Observable):
         emitted when the availability of :meth:`redo` changes
 
     """
+    _in_undo = util.Switch()
+    _in_redo = util.Switch()
+
     undo_redo_enabled = True
 
     def __init__(self, text=""):
@@ -321,7 +382,6 @@ class Document(AbstractDocument, util.Observable):
         self._modified = False
         self._undo_stack = []
         self._redo_stack = []
-        self._in_undo = None
 
     def modified(self):
         """Return whether the text was modified."""
@@ -331,7 +391,7 @@ class Document(AbstractDocument, util.Observable):
         """Sets whether the text is modified, happens automatically normally."""
         changed = modified != self._modified
         self._modified = modified
-        if not modified and not self._in_undo:
+        if not modified and not (self._in_undo or self._in_redo):
             self._set_all_undo_redo_modified()
         if changed:
             self.emit("modification_changed", modified)
@@ -361,11 +421,11 @@ class Document(AbstractDocument, util.Observable):
 
     def _handle_undo(self, start, end, text):
         """Store start, end, and text needed to reconstruct the previous state."""
-        if self._in_undo == "undo":
+        if self._in_undo:
             self._redo_stack.append([start, end, text, self.modified()])
         else:
             self._undo_stack.append([start, end, text, self.modified()])
-            if self._in_undo != "redo":
+            if not self._in_redo:
                 self._redo_stack.clear()
 
     @contextlib.contextmanager
@@ -396,21 +456,19 @@ class Document(AbstractDocument, util.Observable):
         """Undo the last modification."""
         assert self._edit_context == 0, "can't undo while in edit context"
         if self._undo_stack:
-            self._in_undo = "undo"
-            start, end, text, modified = self._undo_stack.pop()
-            self[start:end] = text
-            self.set_modified(modified)
-            self._in_undo = None
+            with self._in_undo:
+                start, end, text, modified = self._undo_stack.pop()
+                self[start:end] = text
+                self.set_modified(modified)
 
     def redo(self):
         """Redo the last undone modification."""
         assert self._edit_context == 0, "can't redo while in edit context"
         if self._redo_stack:
-            self._in_undo = "redo"
-            start, end, text, modified = self._redo_stack.pop()
-            self[start:end] = text
-            self.set_modified(modified)
-            self._in_undo = None
+            with self._in_redo:
+                start, end, text, modified = self._redo_stack.pop()
+                self[start:end] = text
+                self.set_modified(modified)
 
     def clear_undo_redo(self):
         """Clear the undo/redo stack."""
@@ -419,17 +477,17 @@ class Document(AbstractDocument, util.Observable):
             self._redo_stack.clear()
 
     def can_undo(self):
-        """Return True whether undo is possible."""
+        """Return True if undo is possible."""
         return bool(self._undo_stack)
 
     def can_redo(self):
-        """Return True whether redo is possible."""
+        """Return True if redo is possible."""
         return bool(self._redo_stack)
 
     def contents_changed(self, position, removed, added):
-        """Called by _apply().
+        """Called by ``_apply_changes()``.
 
-        This implementation emit ``"contents_change"`` and
+        The default implementation emits the ``"contents_change"`` and
         ``"contents_changed"`` events.
 
         """
@@ -437,28 +495,105 @@ class Document(AbstractDocument, util.Observable):
         self.emit("contents_changed")
 
 
-class Cursor:
+class AbstractTextRange:
+    """Base class for Cursor and Block.
+
+    The text range is denoted by the ``pos`` and ``end`` attributes.
+
+    Provides the comparison operators ``==``, ``!=``, ``>``, ``<``, ``>=``,
+    ``<=``, based on the ``pos`` attribute. The ranges must refer to the same
+    Document.
+
+    """
+    __slots__ = ("_document", "pos", "end")
+    __hash__ = object.__hash__
+
+    def __init__(self, document, pos, end):
+        self._document = document
+        self.pos = pos  #: the (start) position.
+        self.end = end  #: the end position (for Cursor, this may be None).
+
+    def __repr__(self):
+        key = [self.pos]
+        if self.pos != self.end:
+            key.append(self.end or "")
+        key = ":".join(map(format, key))
+        text = reprlib.repr(self.text())
+        return "<{} [{}] {}>".format(type(self).__name__, key, text)
+
+    def document(self):
+        """Return our document."""
+        return self._document
+
+    def text(self):
+        """Return text in this range."""
+        return self.document()[self]
+
+    def __bool__(self):
+        return True
+
+    def __eq__(self, other):
+        """Return ``self.pos == other.pos and self.end == other.end``."""
+        return other.document() is self.document() \
+            and other.pos == self.pos \
+            and other.end == self.end
+
+    def __ne__(self, other):
+        """Return ``self.pos != other.pos or self.end != other.end``."""
+        return other.document() is not self.document() \
+            or other.pos != self.pos \
+            or other.end != self.end
+
+    def __gt__(self, other):
+        """Return ``self.pos > other.pos``."""
+        return self.pos > other.pos
+
+    def __lt__(self, other):
+        """Return ``self.pos < other.pos``."""
+        return self.pos < other.pos
+
+    def __ge__(self, other):
+        """Return ``self.pos >= other.pos``."""
+        return self.pos >= other.pos
+
+    def __le__(self, other):
+        """Return ``self.pos <= other.pos``."""
+        return self.pos <= other.pos
+
+    def token(self):
+        """Convenience method returning the Token at our pos."""
+        return self.document().token(self.pos)
+
+    def tokens(self):
+        """Convenience method returning a tuple with all Tokens that are in
+        or overlap this text range.
+
+        The Document must have the :class:`~parce.treedocument.TreeDocument`
+        class mixed in (i.e. have the ``get_root()`` method.
+
+        """
+        root = self.document().get_root(True)
+        return tuple(root.tokens_range(self.pos, self.end))
+
+
+class Cursor(AbstractTextRange):
     """Describes a certain range (selection) in a Document.
 
-    You may change the start and end attributes yourself. Both must be an
+    You may change the ``pos`` and ``end`` attributes yourself. Both must be an
     integer, end may also be None, denoting the end of the document.
 
     As long as you keep a reference to the Cursor, its positions are updated
-    when the document changes. When text is inserted at the start position, the
-    position remains the same. But when text is inserted at the end of a
-    cursor, the end position moves along with the new text. E.g.:
-
-    .. code-block:: python
+    when the document changes. When text is inserted at ``pos``, the position
+    remains the same. But when text is inserted at the end of a cursor, the
+    ``end`` position (if not None) moves along with the new text. E.g.::
 
         d = Document('hi there, folks!')
         c = Cursor(d, 8, 8)
         with d:
             d[8:8] = 'new text'
-        c.start, c.end --> (8, 16)
+        c.pos, c.end --> (8, 16)
 
-    You can also use a Cursor as key while editing a document:
-
-    .. code-block:: python
+    You can also use a Cursor as key while editing a document::
 
         c = Cursor(d, 8, 8)
         with d:
@@ -467,53 +602,51 @@ class Cursor:
     You cannot alter the document via the Cursor.
 
     """
-    __slots__ = "_document", "end", "start", "__weakref__"
+    __slots__ = ("__weakref__",)
 
-    def __init__(self, document, start=0, end=-1):
-        """Init with document. Start default to 0 and end defaults to start."""
-        self._document = document
-        self.start = start
-        self.end = end if end != -1 else start
+    def __init__(self, document, pos=0, end=-1):
+        """Init with document. ``pos`` defaults to 0 and ``end`` defaults to pos."""
+        super().__init__(document, pos, end if end != -1 else pos)
         document._cursors.add(self)
 
-    def __repr__(self):
-        key = [self.start]
-        if self.start != self.end:
-            key.append(self.end or "")
-        key = ":".join(map(format, key))
-        text = self.text()
-        if len(text) > 30:
-            text = text[:28] + "..."
-        return "<{} [{}] {}>".format(type(self).__name__, key, repr(text))
+    def block(self):
+        """Return the Block our ``pos`` is in."""
+        return self.document().find_block(self.pos)
 
-    def document(self):
-        return self._document
+    def blocks(self):
+        """Yield the Blocks from pos to end."""
+        yield from self.document().blocks(self.pos, self.end)
 
-    def text(self):
-        """Return the selected text, if any."""
-        return self._document[self]
-
-    def select(self, start, end=-1):
-        """Change start and end in one go. End defaults to start."""
-        self.start = start
-        self.end = start if end == -1 else end
+    def select(self, pos, end=-1):
+        """Change pos and end in one go. End defaults to pos."""
+        self.pos = pos
+        self.end = pos if end == -1 else end
 
     def select_all(self):
-        """Set start to 0 and end to None; selecting all text."""
-        self.start = 0
+        """Set pos to 0 and end to None; selecting all text."""
+        self.pos = 0
         self.end = None
 
     def select_none(self):
-        """Set end to start."""
-        self.end = self.start
+        """Set end to pos."""
+        self.end = self.pos
 
     def has_selection(self):
         """Return True if text is selected."""
-        end = len(self._document) if self.end is None else self.end
-        return self.start < end
+        end = len(self.document()) if self.end is None else self.end
+        return self.pos < end
+
+    def select_start_of_block(self):
+        """Moves the selection pos to the beginning of the current line."""
+        self.pos = self.document().find_start_of_block(self.pos)
+
+    def select_end_of_block(self):
+        """Moves the selection end (if not None) to the end of its line."""
+        if self.end is not None:
+            self.end = self.document().find_end_of_block(self.end)
 
     def lstrip(self, chars=None):
-        """Move start to the right, if specified characters can be skipped.
+        """Move pos to the right, if specified characters can be skipped.
 
         By default whitespace is skipped, like Python's lstrip() string method.
 
@@ -521,7 +654,7 @@ class Cursor:
         text = self.text()
         if text:
             offset = len(text) - len(text.lstrip(chars))
-            self.start += offset
+            self.pos += offset
 
     def rstrip(self, chars=None):
         """Move end to the left, if specified characters can be skipped.
@@ -533,13 +666,58 @@ class Cursor:
         if text:
             offset = len(text) - len(text.rstrip(chars))
             if offset:
-                if self.end is None or self.end > len(self._document):
-                    self.end = len(self._document)
+                doc_length = len(self.document())
+                if self.end is None or self.end > doc_length:
+                    self.end = doc_length
                 self.end -= offset
 
     def strip(self, chars=None):
-        """Adjust start and end, like Python's strip() method."""
+        """Adjust pos and end, like Python's strip() method."""
         self.rstrip(chars)
         self.lstrip(chars)
 
 
+
+class Block(AbstractTextRange):
+    r"""Represents a single line (block) of text in the Document.
+
+    Block objects are separated by newlines in the Document, and are created
+    by Document.find_block() or Cursor.block(), and the blocks() iterator of
+    both Cursor and Document.
+
+    Unlike Cursor, Block objects do not update their position when the document
+    is changed. You should use Blocks while iterating but throw them away after
+    applying changes to a Document.
+
+    Blocks can be compared: blocks originating from the same document compare
+    equal when they point to the same position. You can also use the ``<``,
+    ``<=``, ``>`` and ``>=`` operators.
+
+    """
+    __slots__ = ()
+
+    def __len__(self):
+        return self.end - self.pos
+
+    def is_first(self):
+        """True if this is the first block."""
+        return self.pos == 0
+
+    def is_last(self):
+        """True if this is the last block."""
+        return self.end >= len(self.document())
+
+    def next_block(self):
+        """The next block if available."""
+        if not self.is_last():
+            pos = self.end + len(self.document().block_separator)
+            end = self.document().find_end_of_block(pos)
+            return type(self)(self.document(), pos, end)
+
+    def previous_block(self):
+        """The previous block if available."""
+        if self.pos > 0:
+            end = self.pos - len(self.document().block_separator)
+            pos = self.document().find_start_of_block(end)
+            return type(self)(self.document(), pos, end)
+
