@@ -44,6 +44,14 @@ register("parce.lang.css.Css.root",
     guesses = [(r'\b@media\b', 0.5), (r'\bdiv\b', 0.1), (r'\bbody\s*\{', 0.4)],
 )
 
+register("parce.lang.docbook.DocBook.root",
+    name = "DocBook",
+    desc = "DocBook XMl/SGML computer documentation schema",
+    filenames = [("*.docbook", 1), ("*.dbk", 1), ("*.xml", .1)],
+    mimetypes = [("text/docbook", 1), ("application/sgml", .1)],
+    guesses = [(r'^\s*<\?xml ', .2), (r'OASIS.*?DocBook', 1), (r'<book\b', .8)],
+)
+
 register("parce.lang.xml.Dtd.root",
     name = "DTD",
     desc = "Document Type Definition",
@@ -111,6 +119,20 @@ register("parce.lang.python.Python.root",
     guesses = [(r'^#!.{,20}python', .8), (r'\bimport\s+[a-z]+\b', .3)],
 )
 
+register("parce.lang.python.PythonConsole.root",
+    name = "Python Console",
+    desc = "Python console session",
+    guesses = [(r'^>>> ', .3)],
+)
+
+register("parce.lang.tcl.Tcl.root",
+    name = "Tcl",
+    desc = "Tool command language",
+    filenames = [("*.tcl", 1)],
+    mimetypes = [("text/tcl", .8), ("text/x-tcl", .8), ("text/x-script.tcl", .8)],
+    guesses = [(r'^#!.*?(wi|tcl)sh', .8), (r'^namespace eval', .1)],
+)
+
 register("parce.lang.texinfo.Texinfo.root",
     name = "Texinfo",
     desc = "GNU Texinfo",
