@@ -28,7 +28,7 @@ highlighting formats in css files.
 
 """
 
-__all__ = ('Css', 'CssTransform', 'Rule', 'Atrule', 'Color', 'Value')
+__all__ = ('Css', 'CssTransform', 'CssIndent', 'Rule', 'Atrule', 'Color', 'Value')
 
 import collections
 import re
@@ -39,6 +39,7 @@ from parce.action import (
     Number, Operator, String)
 from parce.rule import TEXT, bygroup, ifmember, ifeq, anyof
 from parce.transform import Transform
+from parce.indent import Indent, INDENT, DEDENT
 
 
 RE_CSS_ESCAPE = r"\\(?:[0-9A-Fa-f]{1,6} ?|.)"
@@ -886,3 +887,12 @@ class Value:
         return '<{} {}>'.format(self.__class__.__name__, ', '.join(gen()))
 
 
+class CssIndent(Indent):
+    """Indenter for Css."""
+    def indent_events(self, block, tokens, is_first, prev_indents):
+        for t in tokens:
+            if t.action is Bracket:
+                if t == "{":
+                    yield INDENT,
+                elif t == "}":
+                    yield DEDENT,
