@@ -97,10 +97,9 @@ class LilyPond(Language):
     def root(cls):
         """Toplevel LilyPond document."""
         yield from cls.blocks()
-        yield RE_LILYPOND_SYMBOL, Name.Variable.Definition
-        yield "[,.]", Delimiter
+        yield RE_LILYPOND_SYMBOL, Name.Variable.Definition, cls.identifier
+        yield from cls.find_string(cls.identifier)
         yield "=", Operator.Assignment
-        yield r"\\version\b", Keyword
         yield from cls.music()
 
     @classmethod
@@ -122,39 +121,43 @@ class LilyPond(Language):
 
     @lexicon(consume=True)
     def score(cls):
+        """A score block, can also occur inside markup."""
         yield r'\}', Bracket.End, -1
         yield from cls.blocks()
         yield from cls.music()
 
     @lexicon(consume=True)
     def header(cls):
+        """A header block."""
         yield r'\}', Bracket.End, -1
-        yield RE_LILYPOND_SYMBOL, Name.Variable
-        yield "[,.]", Delimiter
+        yield RE_LILYPOND_SYMBOL, Name.Variable, cls.identifier
         yield "=", Operator.Assignment
         yield from cls.common()
 
     @lexicon(consume=True)
     def paper(cls):
+        """A paper block."""
         yield r'\}', Bracket.End, -1
-        yield RE_LILYPOND_SYMBOL, Name.Variable
-        yield "[,.]", Delimiter
+        yield RE_LILYPOND_SYMBOL, Name.Variable, cls.identifier
         yield "=", Operator.Assignment
-        yield RE_FRACTION + r"|\d+", Number
+        yield r'\d+', Number, cls.unit
+        yield RE_FRACTION, Number
         yield from cls.common()
         yield from cls.commands()
 
     @lexicon(consume=True)
     def layout(cls):
+        """A layout block."""
         yield r'\}', Bracket.End, -1
-        yield RE_LILYPOND_SYMBOL, Name.Variable
-        yield "[,.]", Delimiter
+        yield RE_LILYPOND_SYMBOL, Name.Variable, cls.identifier
         yield "=", Operator.Assignment
+        yield r'\d+', Number, cls.unit
         yield r"(\\context)\s*(\{)", bygroup(Keyword, Bracket.Start), cls.layout_context
         yield from cls.music()
 
     @lexicon(consume=True)
     def midi(cls):
+        """A midi block."""
         yield from cls.layout
 
     @lexicon(consume=True)
@@ -164,8 +167,9 @@ class LilyPond(Language):
         yield words(lilypond_words.contexts, prefix=r"\\", suffix=r"\b"), Context
         yield words(lilypond_words.grobs), Grob
         yield words(lilypond_words.keywords, prefix=r"\\", suffix=r"(?![^\W\d])"), Keyword
-        yield RE_LILYPOND_SYMBOL, Name.Variable, cls.varname
-        yield r"(=)(?:\s*(" + RE_FRACTION + r"|\d+))?", bygroup(Operator.Assignment, Number)
+        yield RE_LILYPOND_SYMBOL, Name.Variable, cls.identifier
+        yield "=", Operator.Assignment
+        yield r'\d+', Number, cls.unit
         yield from cls.common()
         yield from cls.commands()
 
@@ -192,7 +196,7 @@ class LilyPond(Language):
         yield from cls.notemode_rule()
         yield from cls.lyricmode_rules()
         yield from cls.drummode_rule()
-        yield RE_LILYPOND_COMMAND, findmember(MATCH(1), (
+        yield RE_LILYPOND_COMMAND, findmember(MATCH[1], (
             (lilypond_words.keywords, Keyword),
             (lilypond_words.music_commands, Name.Builtin),
             (lilypond_words.all_articulations, Articulation),
@@ -268,7 +272,7 @@ class LilyPond(Language):
         yield words(lilypond_words.contexts), Context
         yield r'[.,]', Delimiter
         yield r"(=)(?:\s*(" + RE_FRACTION + r"|\d+))?", bygroup(Operator.Assignment, Number)
-        yield RE_LILYPOND_SYMBOL + r"(?=\s*([,.=])?)", Name.Variable.Definition, ifeq(MATCH(1), None, -1)
+        yield RE_LILYPOND_SYMBOL + r"(?=\s*([,.=])?)", Name.Variable.Definition, ifeq(MATCH[1], None, -1)
         yield default_target, -1
 
     @lexicon(consume=True)
@@ -279,13 +283,14 @@ class LilyPond(Language):
         yield words(lilypond_words.grobs), Grob
         yield r'[.,]', Delimiter
         yield r"=", Operator.Assignment, -1
-        yield RE_LILYPOND_SYMBOL + r"(?=\s*([,.=])?)", Name.Variable, ifeq(MATCH(1), None, -1)
-        yield from cls.scheme()
+        yield RE_LILYPOND_SYMBOL + r"(?=\s*([,.=])?)", Name.Variable, ifeq(MATCH[1], None, -1)
+        yield from cls.find_scheme()
         yield default_target, -1
 
     # ------------------ script -------------------------
     @lexicon
     def script(cls):
+        """A script abbreviation or fingering digit."""
         yield r"[+|!>._^-]", Script, -1
         yield r"\d+", Fingering, -1
         yield default_target, -1
@@ -302,6 +307,7 @@ class LilyPond(Language):
 
     @lexicon
     def pitch(cls):
+        """A note name, find octave/accidental etc after it."""
         yield r",+|'+", Octave
         yield r"[?!]", Accidental
         yield r"=(,+|'+)?", OctaveCheck, -1
@@ -311,17 +317,17 @@ class LilyPond(Language):
     # ------------------ duration ------------------------
     @lexicon
     def duration_dots(cls):
-        """ zero or more dots after a duration. """
+        """Zero or more dots after a duration. """
         yield SKIP_WHITESPACE
         yield r'\.', Duration.Dot
-        #yield from cls.comments()
+        #yield from cls.find_comment()
         yield default_target, cls.duration_scaling
 
     @lexicon
     def duration_scaling(cls):
         """ * n / m after a duration. """
         yield SKIP_WHITESPACE
-        #yield from cls.comments()
+        #yield from cls.find_comment()
         yield r"(\*)\s*(\d+(?:/\d+)?)", bygroup(Duration, Duration.Scaling)
         yield default_target, -2
 
@@ -329,9 +335,10 @@ class LilyPond(Language):
     # --------------------- lyrics -----------------------
     @classmethod
     def lyricmode_rules(cls):
+        """Find lyric mode music."""
         yield r"(\\(?:lyric(?:mode|s)|addlyrics))\b\s*(\\s(?:equential|imultaneous)\b)?\s*(\{|<<)?", \
             bygroup(Keyword.Lyric, Keyword, Bracket.Start), \
-            dselect(MATCH(3), {'{': cls.lyricmode('}'), '<<': cls.lyricmode('>>')})
+            dselect(MATCH[3], {'{': cls.lyricmode('}'), '<<': cls.lyricmode('>>')})
         yield r"\\lyricsto\b", Keyword.Lyric, cls.lyricsto
 
     @lexicon(consume=True)
@@ -366,7 +373,7 @@ class LilyPond(Language):
     def notemode_rule(cls):
         """Yield the rule for \\notemode / \\notes."""
         yield r"(\\note(?:s|mode))\b\s*(\{|<<)?", bygroup(Keyword, Bracket.Start), \
-            dselect(MATCH(2), {
+            dselect(MATCH[2], {
                 "{": (cls.notemode, cls.musiclist('}')),
                 "<<": (cls.notemode, cls.musiclist('>>')),
             })
@@ -381,7 +388,7 @@ class LilyPond(Language):
     def drummode_rule(cls):
         """Yield the rule for \\drummode / \\drums."""
         yield r"(\\drum(?:s|mode))\b\s*(\{|<<)?", bygroup(Keyword.Drum, Bracket.Start), \
-            dselect(MATCH(2), {
+            dselect(MATCH[2], {
                 "{": (cls.drummode('}')),
                 "<<": (cls.drummode('>>')),
             })
@@ -420,19 +427,40 @@ class LilyPond(Language):
     def common(cls):
         """Find comment, string, scheme and markup."""
         yield from cls.base()
-        yield r"\\markup(lines|list)?" + RE_LILYPOND_ID_RIGHT_BOUND, Keyword.Markup, cls.markup
+        yield r"\\markup(?:lines|list)?" + RE_LILYPOND_ID_RIGHT_BOUND, Keyword.Markup, cls.markup
 
     @classmethod
     def base(cls):
         """Find comment, string and scheme."""
         yield r'"', String, cls.string
-        yield from cls.scheme()
-        yield from cls.comments()
+        yield from cls.find_scheme()
+        yield from cls.find_comment()
 
     @lexicon(consume=True)
-    def varname(cls):
+    def identifier(cls):
         """bla.bla.bla syntax."""
-        yield r'\s*(\.)\s*(' + RE_LILYPOND_SYMBOL + ')', bygroup(Separator.Dot, Name.Variable)
+        yield SKIP_WHITESPACE
+        yield r'([.,])\s*(\d+)' + RE_LILYPOND_ID_RIGHT_BOUND, bygroup(Separator, Number)
+        yield r'([.,])\s*(' + RE_LILYPOND_ID + r')' + RE_LILYPOND_ID_RIGHT_BOUND, bygroup(Separator, Name.Variable)
+        yield r'([.,])\s*(?=[#$"])', Separator, cls.identifier_scheme_or_string
+        yield default_target, -1
+
+    @lexicon
+    def identifier_scheme_or_string(cls):
+        """Pick a scheme expression or a string.
+
+        Immediately pops back, so this context is never actually created.
+
+        """
+        yield from cls.find_string(-1)
+        yield from cls.find_scheme(-1)
+        yield default_target, -1
+
+    @lexicon(consume=True)
+    def unit(cls):
+        """A unit that might occur after a numeric value in a paper block."""
+        yield SKIP_WHITESPACE
+        yield r'\\(mm|in|pt|cm)' + RE_LILYPOND_ID_RIGHT_BOUND, Name.Builtin, -1
         yield default_target, -1
 
     # -------------------- markup --------------------
@@ -442,10 +470,10 @@ class LilyPond(Language):
         yield r'\{', Bracket.Markup.Start, -1, cls.markuplist
         yield r"(\\score)\s*(\{)", bygroup(Name.Function.Markup, Bracket.Start), -1, cls.score
         yield RE_LILYPOND_COMMAND, cls.get_markup_action(), \
-            select(call(cls.get_markup_argument_count, MATCH(1)), -1, 0, 1, 2, 3)
-        yield r'"', String, -1, cls.string
-        yield from cls.scheme(-1)
-        yield from cls.comments()
+            select(call(cls.get_markup_argument_count, MATCH[1]), -1, 0, 1, 2, 3)
+        yield from cls.find_string(-1)
+        yield from cls.find_scheme(-1)
+        yield from cls.find_comment()
         yield RE_LILYPOND_MARKUP_TEXT, Text, -1
 
     @classmethod
@@ -469,13 +497,13 @@ class LilyPond(Language):
     @classmethod
     def get_markup_action(cls):
         r"""Get the action for a command in \markup { }."""
-        return ifmember(MATCH(1), lilypond_words.markup_commands, Name.Function.Markup, Name.Function)
+        return ifmember(MATCH[1], lilypond_words.markup_commands, Name.Function.Markup, Name.Function)
 
     # -------------- Scheme ---------------------
     @classmethod
-    def scheme(cls, pop=0):
+    def find_scheme(cls, extra_target=0):
         """Find scheme."""
-        yield r'[#$]', Delimiter.ModeChange.SchemeStart, pop, cls.get_scheme_target()
+        yield r'[#$]', Delimiter.ModeChange.SchemeStart, extra_target, cls.get_scheme_target()
 
     @classmethod
     def get_scheme_target(cls):
@@ -490,29 +518,34 @@ class LilyPond(Language):
         yield from cls.root()
 
     # -------------- String ---------------------
+    @classmethod
+    def find_string(cls, extra_target=0):
+        """Find a string."""
+        yield '"', String, extra_target, cls.string
+
     @lexicon(consume=True)
     def string(cls):
+        """A double-quoted string."""
         yield r'"', String, -1
-        yield from cls.string_common()
-
-    @classmethod
-    def string_common(cls):
         yield r'\\[\\"]', String.Escape
         yield default_action, String
 
     # -------------- Comment ---------------------
     @classmethod
-    def comments(cls):
+    def find_comment(cls):
+        """Find single-line or block comments."""
         yield r'%\{', Comment, cls.multiline_comment
         yield r'%', Comment, cls.singleline_comment
 
     @lexicon(consume=True)
     def multiline_comment(cls):
+        """A multiple line (block) comment."""
         yield r'%}', Comment, -1
         yield from cls.comment_common()
 
     @lexicon(re_flags=re.MULTILINE, consume=True)
     def singleline_comment(cls):
+        """A comment till the end of the line."""
         yield from cls.comment_common()
         yield r'$', Comment, -1
 
