@@ -78,14 +78,28 @@ Which could also be written as::
         lambda t: any(t.query.next.target.children.containing('2')))
 
 
-A query is a generator, you can iterate over the results. For debugging
-purposes, there are also the list(), pick(), count() and dump() methods.::
+A query is a generator, you can iterate over the results::
 
     for attrs in q.all.action(Name.Tag)('origin').right:
         for atr in attrs.query.action(Name.Attribute):
             print(atr)
 
 
+For debugging purposes, there are also the ``list()``, ``pick()``, ``count()``
+and ``dump()`` methods::
+
+    root.query.all.action(Name.Tag)("img").count() # number of "img" tags
+    root.query.all.action(Name.Tag)("img").list()  # list of all "img" tag name tokens
+
+
+Note that a (partial) query can be reused, it simply restarts the iteration
+over the results. The above could also be written as::
+
+    q = root.query.all.action(Name.Tag)("img")
+    q.count()   # number of "img" tags
+    q.list()    # list of all "img" tag name tokens
+
+
 A query resolves to False if there is no single result::
 
     if token.query.ancestors(LilyPond.header):
@@ -104,13 +118,13 @@ Summary of the query methods:
 
 Endpoint methods (some are mainly for debugging):
 
-    :meth:`~Query.count`,
-    :meth:`~Query.dump`,
-    :meth:`~Query.list`,
-    :meth:`~Query.pick`,
-    :meth:`~Query.pick_last`,
-    :meth:`~Query.range` and
-    :meth:`~Query.delete`.
+:meth:`~Query.count`,
+:meth:`~Query.dump`,
+:meth:`~Query.list`,
+:meth:`~Query.pick`,
+:meth:`~Query.pick_last`,
+:meth:`~Query.range` and
+:meth:`~Query.delete`.
 
 
 Navigating nodes:
@@ -121,25 +135,23 @@ following methods you can find your way through a tree structure. Every method
 returns a new Query object, having the previous one as source of nodes. Most
 methods are implemented as properties, so you don't have to write parentheses.
 
-    :attr:`~Query.all`,
-    :attr:`~Query.children`,
-    :attr:`~Query.parent`,
-    :attr:`~Query.ancestors`,
-    :attr:`~Query.next`,
-    :attr:`~Query.previous`,
-    :attr:`~Query.forward`,
-    :attr:`~Query.backward`,
-    :attr:`~Query.right`,
-    :attr:`~Query.left`,
-    :attr:`~Query.right_siblings`,
-    :attr:`~Query.left_siblings`,
-    :attr:`[n] <Query.__getitem__>`,
-    :attr:`[n:m] <Query.__getitem__>`,
-    :attr:`~Query.first`,
-    :attr:`~Query.last`,
-    :attr:`~Query.target`,
-    :attr:`~Query.source`, and
-    :meth:`~Query.map`,
+:attr:`~Query.all`,
+:attr:`~Query.children`,
+:attr:`~Query.parent`,
+:attr:`~Query.ancestors`,
+:attr:`~Query.next`,
+:attr:`~Query.previous`,
+:attr:`~Query.forward`,
+:attr:`~Query.backward`,
+:attr:`~Query.right`,
+:attr:`~Query.left`,
+:attr:`~Query.right_siblings`,
+:attr:`~Query.left_siblings`,
+:attr:`[n] <Query.__getitem__>`,
+:attr:`[n:m] <Query.__getitem__>`,
+:attr:`~Query.first`,
+:attr:`~Query.last`, and
+:meth:`~Query.map`,
 
 
 Selecting (filtering) nodes:
@@ -148,13 +160,13 @@ Selecting (filtering) nodes:
 These methods filter out current nodes without adding new nodes
 to the selection:
 
-    :attr:`~Query.tokens`,
-    :attr:`~Query.contexts`,
-    :attr:`~Query.uniq`,
-    :attr:`~Query.remove_ancestors`,
-    :attr:`~Query.remove_descendants`,
-    :meth:`~Query.slice` and
-    :meth:`~Query.filter`.
+:attr:`~Query.tokens`,
+:attr:`~Query.contexts`,
+:attr:`~Query.uniq`,
+:attr:`~Query.remove_ancestors`,
+:attr:`~Query.remove_descendants`,
+:meth:`~Query.slice` and
+:meth:`~Query.filter`.
 
 The special :attr:`~Query.is_not` operator inverts the meaning of the
 next query, e.g.::
@@ -163,18 +175,18 @@ next query, e.g.::
 
 The following query methods can be inverted by prepending `is_not`:
 
-    :meth:`~Query.len`,
-    :meth:`~Query.in_range`,
-    :meth:`(lexicon) <Query.__call__>`,
-    :meth:`(lexicon, lexicon2, ...) <Query.__call__>`,
-    :meth:`("text") <Query.__call__>`,
-    :meth:`("text", "text2", ...) <Query.__call__>`,
-    :meth:`~Query.startingwith`,
-    :meth:`~Query.endingwith`,
-    :meth:`~Query.containing`,
-    :meth:`~Query.matching`,
-    :meth:`~Query.action` and
-    :meth:`~Query.in_action`.
+:meth:`~Query.len`,
+:meth:`~Query.in_range`,
+:meth:`(lexicon) <Query.__call__>`,
+:meth:`(lexicon, lexicon2, ...) <Query.__call__>`,
+:meth:`("text") <Query.__call__>`,
+:meth:`("text", "text2", ...) <Query.__call__>`,
+:meth:`~Query.startingwith`,
+:meth:`~Query.endingwith`,
+:meth:`~Query.containing`,
+:meth:`~Query.matching`,
+:meth:`~Query.action` and
+:meth:`~Query.in_action`.
 
 There is a subtle difference between `action` and `in_action`: with the
 first, the action should exactly match, with the latter the tokens are
@@ -506,32 +518,6 @@ class Query:
         for n in self:
             yield from n.left_siblings()
 
-    @pquery
-    def target(self):
-        """Yield the target Context for every token, if available.
-
-        See :meth:`Token.target() <parce.tree.Token.target>`.
-
-        """
-        for t in self:
-            if t.is_token:
-                target = t.target()
-                if target:
-                    yield target
-
-    @pquery
-    def source(self):
-        """Yield the source Token for every context, if available.
-
-        See :meth:`Context.source() <parce.tree.Context.source>`.
-
-        """
-        for n in self:
-            if n.is_context:
-                source = n.source()
-                if source:
-                    yield source
-
     @query
     def map(self, function):
         """Call the function on every node and yield its results, which should be zero or more nodes as well."""
@@ -686,8 +672,9 @@ class Query:
         unless you use ^ or $ characters).
 
         """
+        search = re.compile(pattern, flags).search
         for t in self:
-            if t.is_token and self._inv ^ bool(re.search(pattern, t.text, flags)):
+            if t.is_token and self._inv ^ bool(search(t.text)):
                 yield t
 
     @query
