@@ -22,8 +22,12 @@
 This module contains the implementation of the replacable rule item objects,
 and some functions to query and manipulate rules that are used by the Lexicon.
 
+Normally, you don't need this module directly; use the :mod:`~parce.rule`
+module for your language definitions.
+
 """
 
+import operator
 
 from . import util
 
@@ -53,10 +57,11 @@ class Item:
 
     """
     __slots__ = ()
+    _getitem_func = operator.getitem
 
     def __getitem__(self, n):
         """Return a new Item that performs item[n]. n is evaluated as well."""
-        return call(_get_item, self, n)
+        return call(self._getitem_func, self, n)
 
     def evaluate(self, ns):
         """Evaluate item in namespace dict ``ns``."""
@@ -106,11 +111,6 @@ class Item:
         return ()
 
 
-# helper function used for Item.__getitem__
-def _get_item(text, n):
-    return text[n]
-
-
 class RuleItem(Item):
     """Base class for items that may become visible in rules."""
     __slots__ = ()
@@ -119,17 +119,15 @@ class RuleItem(Item):
 class VariableItem(Item):
     """A named variable that's accessed in the namespace.
 
-    If a predicate is given, it is called when this item is called, with this
-    item as first argument, and then the other arguments that were specified.
+    If a getitem_func is given, it is called when this item is accessed using
+    the item syntax ``[n]``, with this item as first argument, and then the
+    specified index.
 
     """
-    __slots__ = ('_name', '_predicate')
-    def __init__(self, name, predicate=None):
+    __slots__ = ('_name', '_getitem_func')
+    def __init__(self, name, getitem_func=operator.getitem):
         self._name = name
-        self._predicate = predicate
-
-    def __call__(self, *args):
-        return call(self._predicate, self, *args)
+        self._getitem_func = getitem_func
 
     def evaluate(self, ns):
         """Get the variable from the namespace dict."""
@@ -415,7 +413,7 @@ class DelegateAction(ActionItem):
         """Use our lexicon to parse the matched text."""
         sublexer = type(lexer)([self._lexicon])
         for e in sublexer.events(text):
-            for p, txt, action in e.tokens:
+            for p, txt, action in e.lexemes:
                 yield pos + p, txt, action
 
     def evaluate_items(self):
