@@ -29,22 +29,23 @@ By default, the properties are read from a normal CSS (Cascading StyleSheets)
 file and presented to the user of the Theme module through TextFormat objects,
 although other storage backends could be devised.
 
-A Theme provides a ``textformat()`` for standard actions, and for three
-general situations: ``window()``, which denotes an editor window (or an
-encompassing DIV or PRE block in HTML), ``selection()``, which is used for
-selected text, and ``currentline()``, which can highlight the current line
-the cursor is in in an editor.
+A Theme provides a ``textformat()`` for standard actions, and a
+``baseformat()`` for general roles, such as ``"window"``, which denotes an
+editor window (or an encompassing DIV or PRE block in HTML), ``"selection"``,
+which is used for selected text, ``"current-line"``, which can highlight the
+current line the cursor is in in an editor.
 
-From the TextFormat returned by ``selection()`` and ``currentline()``, in
+From the TextFormat returned by ``baseformat("selection")`` and ``"currentline"``, in
 most cases only the background color will be used.
 
-The methods ``window()``, ``selection()`` and ``currentline()`` also accept a
-state argument, which can be "default", "focus" or "disabled". A theme always
-supports the default state, but can provide separate colors for the "focus"
-or "disabled" state, which can be used to change the basic formatting in an
+For the roles ``"window"``, ``"selection"`` and ``"current-line"``, the
+``baseformat()`` method also accepts a state argument, which can be
+``"default"``, ``"focus"`` or ``"disabled"``. A theme always supports the
+``"default"`` state, but can provide separate colors for the ``"focus"`` or
+``"disabled"`` state, which can be used to change the basic formatting in an
 editor window based on its state (in keyboard focus or disabled). If a theme
-does not support the "disabled" and/or "focus" state, the default scheme is
-used.
+does not support the ``"disabled"`` and/or ``"focus"`` state, the default
+scheme is used.
 
 In the ``themes/`` directory are bundled CSS themes that can be used.
 Instantiate a bundled theme with::
@@ -66,19 +67,19 @@ Get a TextFormat for an action, use e.g.::
 Mapping actions to CSS classes
 ------------------------------
 
-Standard actions are mapped to a tuple of CSS class names: the action itself
-and the actions it descends from. All CSS rules are combined, the one with the
-most matches comes first.
+Standard actions are mapped to one or more CSS class names using
+:func:`css_class`; it uses the action itself and the actions it descends from.
+All CSS rules are combined, the one with the most matches comes first.
 
-For example, Comment maps to the "comment" CSS class, and Number maps
-to ("literal", "number") because Number is a descendant action of Literal.
+For example, ``Comment`` maps to the ``"comment"`` CSS class, and ``Number``
+maps to ``"literal number"`` because Number is a descendant action of Literal.
 
-Some actions might have the same name, e.g. Escape and String.Escape.
-Both match CSS rules with the ``.escape`` class selector, but a rule
-with ``.string.escape`` will have higher precedence.
+Some actions might have the same name, e.g. ``Escape`` and ``String.Escape``.
+Both match CSS rules with the ``.escape`` class selector, but a rule with
+``.string.escape`` will have a higher precedence.
 
-The order of the action names does not matter. E.g. an action Text.Comment
-will match exactly the same CSS rules as an action Comment.Text. So you
+The order of the action names does not matter. E.g. an action ``Text.Comment``
+will match exactly the same CSS rules as an action ``Comment.Text``. So you
 should take some care when designing you action hierachy and not add too much
 base action types.
 
@@ -94,8 +95,21 @@ from . import themes
 from . import util
 
 
-class Theme:
+class AbstractTheme:
+    """Defines the interface of a Theme as used by a formatter."""
+
+    def baseformat(self, role="window", state="default"):
+        """Should return a text format for a specific role and a state."""
+        raise NotImplementedError
+
+    def textformat(self, action):
+        """Should return a text format for the specified action."""
+        raise NotImplementedError
+
+
+class Theme(AbstractTheme):
     """A Theme maps a StandardAction to a TextFormat with CSS properties."""
+
     def __init__(self, filename="", stylesheet=""):
         """Instantiate Theme from a CSS file or text.
 
@@ -155,8 +169,8 @@ class Theme:
         ``"current-line"``
             The TextFormat for the current line. If you use it, set only the
             *background* color in your theme file.
-        ``"trialing-whitespace"``
-            The TextFormat (use only the *background*) to highlight trialing
+        ``"trailing-whitespace"``
+            The TextFormat (use only the *background*) to highlight trailing
             whitespace, if desired.
         ``"eol-marker"``
             The *color* to draw a "end-of-line" marker with, if desired
@@ -178,108 +192,10 @@ class Theme:
     @util.cached_method
     def textformat(self, action):
         """Return the TextFormat for the specified action."""
-        class_ = repr(action).lower().replace('.', ' ')
+        class_ = css_class(action)
         e = css.Element(class_=class_, parent=css.Element(class_="parce"))
         return self.TextFormat(self.style.select_element(e).properties())
 
-    def tokens(self, theme_context, tree, start=0, end=None):
-        """Yield tokens from ``tree.tokens_range(start, end)``.
-
-        If a context is entered, ``theme_context.push(language)`` is called
-        with the language of the context's lexicon. If the context is left,
-        ``theme_context.pop()`` is called.
-
-        In Theme, the ``theme_context`` argument is unused.
-
-        """
-        yield from tree.tokens_range(start, end)
-
-
-class MetaTheme:
-    """A Theme that encapsulates a default Theme and per-language sub-Themes."""
-    def __init__(self, theme):
-        """Instantiate with default theme."""
-        self._theme = theme
-        self._themes = {}
-        self._add_window = {theme: False}
-
-    def add_theme(self, language, theme, add_window=False):
-        """Add a specific Theme for the specified Language.
-
-        If ``add_window`` is set to True, the formatter will render text from
-        the specified language with its own ``window`` default style added to
-        all formats. A formatter will also need to take care to render
-        untokenized ranges with the ``window()`` style of the sub-theme.
-
-        """
-        self._themes[language] = theme
-        self._add_window[theme] = add_window
-
-    def get_theme(self, language):
-        """Return the tuple(theme, add_window) for the language.
-
-        If the exact language was not found, the base classes of the Language are
-        tried. If still no luck, self is returned.
-
-        """
-        try:
-            return self._themes[language]
-        except KeyError:
-            pass
-        for lang in language.mro()[:-2]:    # don't test Language and object
-            try:
-                return self._themes[lang]
-            except KeyError:
-                pass
-        return self
-
-    def get_add_window(self, theme):
-        """Return the ``add_window`` value that was set when adding this sub-theme."""
-        return self._add_window.get(theme, False)
-
-    def baseformat(self, role="window", state="default"):
-        """Return the base TextFormat for the rold and state of the default theme."""
-        return self._theme.baseformat(state)
-
-    def textformat(self, action):
-        """Return the TextFormat for the specified action of the default theme."""
-        return self._theme.textformat(action)
-
-    def tokens(self, theme_context, tree, start=0, end=None):
-        """Yield tokens from ``tree.tokens_range(start, end)``.
-
-        If a context is entered, ``theme_context.push(language)`` is called
-        with the language of the context's lexicon. If the context is left,
-        ``theme_context.pop()`` is called.
-
-        The theme_context can switch the theme, based on the current language.
-
-        """
-        for context, slice_ in tree.context_slices(start, end):
-            theme_context.push(context.lexicon.language)
-            for n in context[slice_]:
-                stack = []
-                i = 0
-                while True:
-                    for i in range(i, len(n)):
-                        m = n[i]
-                        if m.is_token:
-                            yield m
-                        else:
-                            stack.append(i)
-                            i = 0
-                            n = m
-                            theme_context.push(n.lexicon.language)
-                            break
-                    else:
-                        if stack:
-                            theme_context.pop()
-                            n = n.parent
-                            i = stack.pop() + 1
-                        else:
-                            break
-            theme_context.pop()
-
 
 class TextFormat:
     """Simple textformat that reads CSS properties and supports a subset of those.
@@ -340,7 +256,7 @@ class TextFormat:
     font_variant_position = None    #: normal, sub or super
     font_weight = None              #: 100 - 900 or keyword like ``bold``
 
-    dispatch = util.Dispatcher()
+    _dispatch = util.Dispatcher()
 
     def __repr__(self):
         return "<{} {}>".format(self.__class__.__name__,
@@ -349,7 +265,7 @@ class TextFormat:
 
     def __init__(self, properties):
         for prop, values in properties.items():
-            self.dispatch(prop, values)
+            self._dispatch(prop, values)
 
     def __bool__(self):
         """Return True if at least one property is set."""
@@ -445,32 +361,32 @@ class TextFormat:
         if self.font_weight:
             yield "font-weight", format(self.font_weight)
 
-    @dispatch("color")
+    @_dispatch("color")
     def read_color(self, values):
         for v in values:
             if v.color:
                 self.color = v.color
                 return
 
-    @dispatch("background-color")
+    @_dispatch("background-color")
     def read_background_color(self, values):
         for v in values:
             if v.color:
                 self.background_color = v.color
                 return
 
-    @dispatch("background")
+    @_dispatch("background")
     def read_background(self, values):
         self.read_background_color(values)
 
-    @dispatch("text-decoration-color")
+    @_dispatch("text-decoration-color")
     def read_text_decoration_color(self, values):
         for v in values:
             if v.color:
                 self.text_decoration_color = v.color
                 return
 
-    @dispatch("text-decoration-line")
+    @_dispatch("text-decoration-line")
     def read_text_decoration_line(self, values):
         decos = []
         for v in values:
@@ -480,20 +396,20 @@ class TextFormat:
                 decos.clear()
         self.text_decoration_line = decos
 
-    @dispatch("text-decoration-style")
+    @_dispatch("text-decoration-style")
     def read_text_decoration_style(self, values):
         for v in values:
             if v.text in ("solid", "double", "dotted", "dashed", "wavy"):
                 self.text_decoration_style = v.text
                 return
 
-    @dispatch("text-decoration")
+    @_dispatch("text-decoration")
     def read_text_decoration(self, values):
         self.read_text_decoration_color(values)
         self.read_text_decoration_line(values)
         self.read_text_decoration_style(values)
 
-    @dispatch("font-family")
+    @_dispatch("font-family")
     def read_font_family(self, values):
         families = []
         for v in values:
@@ -511,14 +427,14 @@ class TextFormat:
                 families.append(v.text)
         self.font_family = families
 
-    @dispatch("font-kerning")
+    @_dispatch("font-kerning")
     def read_font_kerning(self, values):
         for v in values:
             if v.text in ("auto", "normal", "none"):
                 self.font_kerning = v.text
                 return
 
-    @dispatch("font-size")
+    @_dispatch("font-size")
     def read_font_size(self, values):
         for v in values:
             if v.text in ("xx-small", "x-small", "small", "medium",
@@ -531,7 +447,7 @@ class TextFormat:
                 self.font_size_unit = v.unit
                 return
 
-    @dispatch("font-stretch")
+    @_dispatch("font-stretch")
     def read_font_stretch(self, values):
         for v in values:
             if v.text in ("ultra-condensed", "extra-condensed", "condensed",
@@ -541,7 +457,7 @@ class TextFormat:
             elif v.number is not None and v.unit == "%":
                 self.font_stretch = v.number
 
-    @dispatch("font-style")
+    @_dispatch("font-style")
     def read_font_style(self, values):
         v = values[0]
         for n in values[1:] + [None]:
@@ -556,7 +472,7 @@ class TextFormat:
                     return
             v = n
 
-    @dispatch("font-variant-caps")
+    @_dispatch("font-variant-caps")
     def read_font_variant_caps(self, values):
         for v in values:
             if v.text in ("normal", "small-caps", "all-small-caps", "petite-caps",
@@ -564,14 +480,14 @@ class TextFormat:
                 self.font_variant_caps = v.text
                 return
 
-    @dispatch("font-variant-position")
+    @_dispatch("font-variant-position")
     def read_font_variant_position(self, values):
         for v in values:
             if v.text in ("normal", "sub", "super"):
                 self.font_variant_position = v.text
                 return
 
-    @dispatch("font-weight")
+    @_dispatch("font-weight")
     def read_font_weight(self, values):
         for v in values:
             if v.text in ("normal", "bold", "lighter", "bolder"):
@@ -581,7 +497,7 @@ class TextFormat:
                 self.font_weight = v.number
                 return
 
-    @dispatch("font")
+    @_dispatch("font")
     def read_font(self, values):
         self.read_font_style(values)
         numvalues = []
@@ -610,3 +526,19 @@ class TextFormat:
             self.font_size, self.font_size_unit = numvalues[1]
 
 
+def css_class(action):
+    """Return a CSS class string for the specified standard action.
+
+    The class names are simply the name of the action and all its ancestor
+    actions. Class names are lowercase and space-separated. For example::
+
+        >>> from parce.action import Number
+        >>> Number
+        Literal.Number
+        >>> css_class(Number)
+        'literal number'
+
+    """
+    return repr(action).lower().replace('.', ' ')
+
+
