@@ -34,7 +34,6 @@ language. But I'm not yet sure how it would be specified.
 """
 
 import collections
-import functools
 import sys
 import threading
 import weakref
@@ -43,43 +42,25 @@ import parce.lexer
 import parce.util
 
 
-class Item:
-    """An Item wraps any object returned by a transform method.
+class Item(collections.namedtuple("Item", "name obj")):
+    """A named tuple(name, obj) wrapping the return value of a Tranform method.
 
-    It has two readonly attributes, ``lexicon`` and ``obj``. The ``lexicon``
-    is the lexicon that created the items a transform method is called with. To
-    be able to easily distinguish an Item from a Token, a fixed readonly class
-    attribute ``is_token`` exists, set to False.
-
-    An Item can be unpacked in two variables, name and object::
-
-        item = Item(Json.root, "abc")
-        name, obj = item    # → 'root', 'abc'
+    The `name` attribute is the name of the Lexicon and transform method that
+    created the object `obj`. To make it easier to distinguish an Item from a
+    Token, Item has as class attribute `is_token` set to False.
 
     """
-    __slots__ = ('lexicon', 'obj')
+    __slots__ = ()
     is_token = False
 
-    def __init__(self, lexicon, obj):
-        self.lexicon = lexicon
-        self.obj = obj
-
-    @property
-    def name(self):
-        """The lexicon's name."""
-        return self.lexicon.name
-
-    def __repr__(self):
-        return "<Item '{}' {}>".format(self.name, repr(self.obj))
-
-    def __iter__(self):
-        yield self.name
-        yield self.obj
-
 
 class Items(list):
     """A list of Item and Token instances.
 
+    The Transformer populates this list with all the Tokens from a Context,
+    replacing nested Contexts with an Item that wraps the return value of the
+    Transform method for that Context's lexicon.
+
     The Transform methods are called with an instance of this class. Besides
     being a Python list, this class also has some methods that filter out items
     or tokens, etc.
@@ -98,7 +79,8 @@ class Items(list):
         return result
 
     def tokens(self, *texts):
-        """Yield only the tokens.
+        """Yield only the tokens, ignoring Item objects that represent
+        sub-contexts.
 
         If one or more texts are given, only yield tokens with one of the
         texts.
@@ -114,7 +96,7 @@ class Items(list):
                     yield i
 
     def items(self, *names):
-        """Yield only the sub-items.
+        """Yield only the sub-Items, ignoring any Token instances.
 
         If one or more names are given, only yield items that have one of the
         names.
@@ -135,75 +117,29 @@ class Items(list):
                 if not i.is_token:
                     yield i
 
-    def objects(self, *classes):
-        """Yield only the objects from sub-items.
+    def grouped_objects(self, *names):
+        """Yield objects in groups, specified by the names.
 
-        If one or mode classes are specified, only yields objects that are
-        an instance of that class/classes.
+        The order remains the same. For example, when you have a stream of `key`
+        and `value` Item objects, calling `grouped_objects('key', 'value')`
+        yields the objects in (key, value) pairs.
+
+        For missing objects, None is yielded. Tokens are ignored.
 
         """
-        if classes:
-            for i in self.items():
-                if isinstance(i.obj, classes):
-                    yield i.obj
-        else:
-            for i in self.items():
-                yield i.obj
+        # a mapping from name to position in the names list
+        index = dict((name, i) for i, name in enumerate(names))
 
-    def action(self, *actions):
-        """Yield only the tokens with one of the specified actions."""
-        for i in self:
-            if i.is_token and i.action in actions:
-                yield i
-
-    def in_action(self, *actions):
-        """Yield only the tokens with an action that's in one of the specified
-        actions.
-
-        """
-        for i in self:
-            if i.is_token and any(i.action in a for a in actions):
-                yield i
-
-    def grouped(self):
-        """Yield two-tuples(tokens, item).
-
-        The ``tokens`` value is a tuple itself of zero or more tokens; grouped
-        tokens (that originated from one match) are yielded together; otherwise
-        there is just one token.
-
-        If the token(s) are followed by an ``item``, it is yielded as well; if
-        not, ``item`` is None. If there are no tokens after an item but there
-        is another item; tokens is an empty tuple then.
-
-        """
-        i = 0
-        z = len(self)
-        get_item = super().__getitem__
-        while i < z:
-            n = self[i]
-            if n.is_token:
-                if n.group is not None:
-                    j = i
-                    for g, j in enumerate(range(i + 1, z), n.group + 1):
-                        m = get_item(j)
-                        if not m.is_token or not m.group or m.group < g:
-                            break
-                    else:
-                        j += 1  # the group was the last
-                    tokens = get_item(slice(i, j))
-                    i = j
-                else:
-                    tokens = n,
-                    i += 1
-                if i < z and not get_item(i).is_token:
-                    yield tokens, get_item(i)
-                    i += 1
-                else:
-                    yield tokens, None
-            else:
-                yield (), n
-                i += 1
+        result = [None] * len(names)
+        lastindex = -1
+        for name, obj in self.items(*names):
+            if index[name] <= lastindex:
+                yield result
+                result = [None] * len(names)
+            lastindex = index[name]
+            result[lastindex] = obj
+        if lastindex > -1:
+            yield result
 
 
 class Transform:
@@ -274,7 +210,7 @@ class Transformer(parce.util.Observable):
                 transform = self.get_transform(curlang)
             name = lexicon.name
             meth = getattr(transform, name, None)
-            return Item(lexicon, meth(items)) if meth else no_object
+            return Item(name, meth(items)) if meth else no_object
 
         for e in events:
             if e.target:
@@ -300,11 +236,11 @@ class Transformer(parce.util.Observable):
 
     def transform_tree(self, tree):
         """Evaluate a tree structure."""
+        self._interrupt[tree] = False
+
         if not tree.lexicon:
             return  # a root lexicon can be None, but then there are no children
 
-        self._interrupt[tree] = False
-
         curlang = tree.lexicon.language
         transform = self.get_transform(curlang)
 
@@ -325,7 +261,7 @@ class Transformer(parce.util.Observable):
                     # don't bother going in this context is there is no method
                     if meth:
                         try:
-                            items.append(Item(n.lexicon, self._cache[n]))
+                            items.append(Item(name, self._cache[n]))
                         except KeyError:
                             stack.append((items, i + 1))
                             node, items, i = n, Items(), 0
@@ -340,7 +276,7 @@ class Transformer(parce.util.Observable):
                 if stack:
                     self._cache[node] = obj
                     items, i = stack.pop()
-                    items.append(Item(node.lexicon, obj))
+                    items.append(Item(name, obj))
                     node = node.parent
                 else:
                     break
@@ -409,7 +345,7 @@ class Transformer(parce.util.Observable):
 
         """
         builder.connect("replace", self.slot_replace, prepend_self=True)
-        builder.connect("finished", self.slot_update, prepend_self=True)
+        builder.connect("finished", self.slot_update, prepend_self=True, priority=-1000)
         builder.connect("invalidate", self.invalidate_node)
 
     def disconnect_treebuilder(self, builder):
@@ -462,7 +398,39 @@ class Transformer(parce.util.Observable):
         module = sys.modules[language.__module__]
         tfname = language.__name__ + "Transform"
         tf = getattr(module, tfname, None)
-        return tf() if issubclass(tf, Transform) else None
+        if isinstance(tf, type) and issubclass(tf, Transform):
+            return tf()
+
+
+class BackgroundTransformer(Transformer):
+    """A Transformer that does its job in a background thread."""
+    def __init__(self):
+        super().__init__()
+        self._jobs = {}
+
+    def build(self, tree):
+        """Reimplemented to build the transformation in a background thread.
+
+        This method starts a background thread performing the transformation
+        and returns immediately.
+
+        """
+        def job():
+            for stage in self.process(tree):
+                pass
+            del self._jobs[tree]
+        job = self._jobs[tree] = threading.Thread(target=job)
+        job.start()
+
+    def wait(self, tree):
+        """Wait for completion of the transformation of the tree if busy."""
+        job = self._jobs.get(tree)
+        if job:
+            job.wait()
+
+    def busy(self, tree):
+        """Return True if a job transforming the tree is busy."""
+        return tree in self._jobs
 
 
 def transform_tree(tree, transform=None):
@@ -512,34 +480,3 @@ def validate_transform(transform, language):
         if not getattr(transform, lexicon.name, None):
             print("Missing transform method for lexicon:", lexicon.name)
 
-
-class BackgroundTransformer(Transformer):
-    """A Transformer that does its job in a background thread."""
-    def __init__(self):
-        super().__init__()
-        self._jobs = {}
-
-    def build(self, tree):
-        """Reimplemented to build the transformation in a background thread.
-
-        This method starts a background thread performing the transformation
-        and returns immediately.
-
-        """
-        def job():
-            for stage in self.process(tree):
-                pass
-            del self._jobs[tree]
-        job = self._jobs[tree] = threading.Thread(target=job)
-        job.start()
-
-    def wait(self, tree):
-        """Wait for completion of the transformation of the tree if busy."""
-        job = self._jobs.get(tree)
-        if job:
-            job.wait()
-
-    def busy(self, tree):
-        """Return True if a job transforming the tree is busy."""
-        return tree in self._jobs
-
