@@ -323,7 +323,7 @@ class Token(Node):
 
     When a pattern rule in a lexicon matches the text, a Token is created. When
     that rule would create more than one Token from a single regular expression
-    match, _GroupToken objects are created instead, carrying the index of the
+    match, GroupToken objects are created instead, carrying the index of the
     token in the group in the `group` attribute. The `group` attribute is
     readonly None for normal tokens.
 
@@ -338,9 +338,11 @@ class Token(Node):
                     break
         pos = t.pos
 
-    (A _GroupToken is just a normal Token otherwise, the reason a subclass was
+    Alternatively, you can use the `GroupToken.get_group_*` methods.
+
+    (A GroupToken is just a normal Token otherwise, the reason a subclass was
     created is that the group attribute is unused in by far the most tokens, so
-    it does not use any memory. You never need to reference the _GroupToken
+    it does not use any memory. You never need to reference the GroupToken
     class; just test the group attribute if you want to know if a token belongs
     to a group that originated from a single match.)
 
@@ -486,33 +488,17 @@ class Token(Node):
                 return n, s_indices[s_i::-1], o_indices[::-1]
         return None, None, None
 
-    def target(self):
-        """Return the first context directly to the right of this Token.
-
-        The context should be the right sibling of the token, or of any of its
-        ancestors. If the token is part of a group, the context is found
-        immediately next to the last member of the group. The found context may
-        also be a child of the grand-parents of this token, in case the target
-        popped contexts first.
-
-        In all cases, the returned context is the one started by the target
-        in the lexicon rule that created this token.
-
-        """
-        node = self
-        if node.group is not None:
-            node = get_group_end(node)
-        while node.parent:
-            r = node.right_sibling()
-            if r:
-                if r.is_context:
-                    return r
-                return
-            node = node.parent
 
 
-class _GroupToken(Token):
-    """A Token class that allows setting the `group` attribute."""
+class GroupToken(Token):
+    """A Token class that allows setting the `group` attribute.
+
+    For normal Token instances, `group` is a class attribute that is always
+    None. For Tokens that belong to a group, i.e. originated from a single
+    regular expression match, the `group` attribute is the index of the token
+    in the group of tokens that were created together.
+
+    """
     __slots__ = "group",
 
     def __init__(self, group, parent, pos, text, action):
@@ -523,6 +509,36 @@ class _GroupToken(Token):
         """Return a copy of the Token, but with the specified parent."""
         return type(self)(self.group, parent, self.pos, self.text, self.action)
 
+    def get_group(self):
+        """Return the whole group this token belongs to as a list."""
+        p = self.parent
+        i = j = self.parent_index()
+        while i and p[i].group > 0 and p[i-1].is_token and p[i-1].group is not None and p[i-1].group < p[i].group:
+            i -= 1
+        z = len(p)
+        j += 1
+        while j < z and p[j].is_token and p[j].group and p[j].group > p[j-1].group:
+            j += 1
+        return p[i:j]
+
+    def get_group_start(self):
+        """Return the first token of the group this token belongs to."""
+        p = self.parent
+        i = self.parent_index()
+        while i and p[i].group > 0 and p[i-1].is_token and p[i-1].group is not None and p[i-1].group < p[i].group:
+            i -= 1
+        return p[i]
+
+    def get_group_end(self):
+        """Return the last token of the group this token belongs to."""
+        p = self.parent
+        j = self.parent_index() + 1
+        z = len(p)
+        while j < z and p[j].is_token and p[j].group and p[j].group > p[j-1].group:
+            j += 1
+        return p[j-1]
+
+
 
 class Context(list, Node):
     """A Context represents a list of tokens and contexts.
@@ -586,12 +602,12 @@ class Context(list, Node):
 
     def __eq__(self, other):
         if isinstance(other, Lexicon):
-            return other.equals(self.lexicon)
+            return self.lexicon == other
         return other is self
 
     def __ne__(self, other):
         if isinstance(other, Lexicon):
-            return not other.equals(self.lexicon)
+            return self.lexicon != other
         return other is not self
 
     def copy(self, parent=None):
@@ -744,40 +760,69 @@ class Context(list, Node):
             pass
 
     def find(self, pos):
-        """Return the index of our child at pos."""
+        """Return the index of our child at (or to the right of) pos.
+
+        Returns -1 if there is no such child.
+
+        """
         i = 0
-        hi = len(self)
+        hi = l = len(self)
         while i < hi:
             mid = (i + hi) // 2
             n = self[mid]
-            if n.pos >= pos:
-                hi = mid
-            elif n.end <= pos:
+            if n.end <= pos:
                 i = mid + 1
             else:
                 hi = mid
-        return min(i, len(self) - 1)
+        return -1 if i == l else i
+
+    def find_context(self, pos):
+        """Return the younghest Context at position (or self)."""
+        node = self
+        i = self.find(pos)
+        if i != -1:
+            n = node[i]
+            while n.is_context and n.pos <= pos:
+                node = n
+                n = n[n.find(pos)]
+        return node
 
     def find_token(self, pos):
-        """Return the Token at or to the right of position."""
-        n = self[self.find(pos)]
-        while n.is_context:
-            n = n[n.find(pos)]
-        return n
+        """Return the Token at or to the right of position.
+
+        Returns None if there is no such token.
+
+        """
+        i = self.find(pos)
+        if i != -1:
+            n = self[i]
+            while n.is_context:
+                n = n[n.find(pos)]
+            return n
 
     def find_token_with_trail(self, pos):
-        """Return the Token at or to the right of position, and the trail of indices."""
+        """Return the Token at or to the right of position, and the trail of indices.
+
+        Returns (None, None) if there is no such token.
+
+        """
         i = self.find(pos)
-        n = self[i]
-        trail = [i]
-        while n.is_context:
-            i = n.find(pos)
-            n = n[i]
-            trail.append(i)
-        return n, trail
+        if i != -1:
+            n = self[i]
+            trail = [i]
+            while n.is_context:
+                i = n.find(pos)
+                n = n[i]
+                trail.append(i)
+            return n, trail
+        return None, None
 
     def find_left(self, pos):
-        """Return the index of our child at or to the left of pos."""
+        """Return the index of our child at or to the left of pos.
+
+        Returns -1 if there is no such child.
+
+        """
         i = 0
         hi = len(self)
         while i < hi:
@@ -787,25 +832,37 @@ class Context(list, Node):
                 i = mid + 1
             else:
                 hi = mid
-        return max(0, i - 1)
+        return i - 1
 
     def find_token_left(self, pos):
-        """Return the Token at or to the left of position."""
-        n = self[self.find_left(pos)]
-        while n.is_context:
-            n = n[n.find_left(pos)]
-        return n
+        """Return the Token at or to the left of position.
+
+        Returns None if there is no such token.
+
+        """
+        i = self.find_left(pos)
+        if i != -1:
+            n = self[i]
+            while n.is_context:
+                n = n[n.find_left(pos)]
+            return n
 
     def find_token_left_with_trail(self, pos):
-        """Return the Token at or to the left of position, and the trail of indices."""
+        """Return the Token at or to the left of position, and the trail of indices.
+
+        Returns (None, None) if there is no such token.
+
+        """
         i = self.find_left(pos)
-        n = self[i]
-        trail = [i]
-        while n.is_context:
-            i = n.find_left(pos)
-            n = n[i]
-            trail.append(i)
-        return n, trail
+        if i != -1:
+            n = self[i]
+            trail = [i]
+            while n.is_context:
+                i = n.find_left(pos)
+                n = n[i]
+                trail.append(i)
+            return n, trail
+        return None, None
 
     def find_token_after(self, pos):
         """Return the first token completely right from pos.
@@ -895,10 +952,14 @@ class Context(list, Node):
             if end <= start:
                 return None, None, None
             end_trail = self.find_token_left_with_trail(end)[1]
+            if not end_trail:
+                return None, None, None
         else:
             end_trail = []
         if start > 0:
             start_trail = self.find_token_with_trail(start)[1]
+            if not start_trail:
+                return None, None, None
             if end_trail:
                 # find the youngest common ancestor
                 for n, (i, j) in enumerate(zip(start_trail, end_trail)):
@@ -955,66 +1016,18 @@ class Context(list, Node):
         else:
             yield self, slice(start, None)
 
-    def source(self):
-        """Return the first Token, if any, when going to the left from this context.
-
-        The returned token is the one that created us, that this context the
-        target is for. If the token is member of a group, the first group member
-        is returned.
-
-        """
-        prev = None
-        for token in self.backward():
-            if not token.group:
-                return token
-            if prev and token.group >= prev.group:
-                return prev
-            prev = token
-
 
 def make_tokens(event, parent=None):
     """Factory returning a tuple of one or more Token instances for the event.
 
     The event is an Event namedtuple defined in the mod:`~parce.lexer` module.
-    If the event contains more than one token, _GroupToken instances are
+    If the event contains more than one lexeme, GroupToken instances are
     created.
 
     """
-    if len(event.tokens) > 1:
-        return tuple(_GroupToken(n, parent, *t) for n, t in enumerate(event.tokens))
+    if len(event.lexemes) > 1:
+        return tuple(GroupToken(n, parent, *t) for n, t in enumerate(event.lexemes))
     else:
-        return Token(parent, *event.tokens[0]),
-
-
-def get_group(token):
-    """For a token that belongs to a group, return the whole group as a list."""
-    p = token.parent
-    i = j = token.parent_index()
-    while i and p[i].group > 0 and p[i-1].is_token and p[i-1].group is not None and p[i-1].group < p[i].group:
-        i -= 1
-    z = len(p)
-    j += 1
-    while j < z and p[j].is_token and p[j].group and p[j].group > p[j-1].group:
-        j += 1
-    return p[i:j]
-
-
-def get_group_start(token):
-    """For a token that belongs to a group, return the first token of the group."""
-    p = token.parent
-    i = token.parent_index()
-    while i and p[i].group > 0 and p[i-1].is_token and p[i-1].group is not None and p[i-1].group < p[i].group:
-        i -= 1
-    return p[i]
-
-
-def get_group_end(token):
-    """For a token that belongs to a group, return the last token of the group."""
-    p = token.parent
-    j = token.parent_index() + 1
-    z = len(p)
-    while j < z and p[j].is_token and p[j].group and p[j].group > p[j-1].group:
-        j += 1
-    return p[j-1]
+        return Token(parent, *event.lexemes[0]),
 
 
