@@ -276,7 +276,7 @@ class TreeBuilder(Observable):
             tail_token = self.root.find_token_after(end)
             if tail_token:
                 tail_gen = ((t, t.pos) for t in tail_token.forward_including()
-                        if not t.group or (t.group and t is t.group[0]))
+                        if not t.group and not (t.is_first() and t.parent.lexicon.consume))
                 for tail_token, tail_pos in tail_gen:
                     tail = True
                     break
@@ -304,8 +304,7 @@ class TreeBuilder(Observable):
                 if result:
                     lexer, events, tokens = result
                     t = tokens[0]
-                    context = new_tree(t)
-                    tree = context.root()
+                    context, tree = new_tree(t)
                     start = tokens[-1].end
                     lowest_start = min(lowest_start, start)
                 else:
@@ -332,7 +331,8 @@ class TreeBuilder(Observable):
                                 break
                         else:
                             tail = False
-                    if pos == tail_pos and tokens[0].equals(tail_token):
+                    if pos == tail_pos and tokens[0].equals(tail_token) and \
+                            (context or not context.lexicon.consume) :
                         # we can reuse the tail from tail_pos
                         return BuildResult(tree, lowest_start, tail_pos, offset, None)
                 context.extend(tokens)
@@ -395,7 +395,7 @@ class TreeBuilder(Observable):
         the tree-replacing process.
 
         Additionally, this method calls :meth:`invalidate_context` with the
-        youghest Context that had children removed or added.
+        youngest Context that had children removed or added.
 
         """
         tree, start, end, offset, lexicons = result
@@ -450,8 +450,13 @@ class TreeBuilder(Observable):
                         t = t[l]    # t can be empty if len(end_trail) == 1, is last iteration anyway
                     c = c[i]
 
-            if offset and slice_end is not None and slice_end < len(context):
-                self.replace_pos(context, slice(slice_end, None), offset)
+            if offset:
+                # if there remain nodes in the current context after tree
+                # insertion, store the index now, later we'll adjust the pos
+                if slice_end is not None:
+                    replace_pos_index = slice_end - len(context)
+                else:
+                    replace_pos_index = 0
 
             if start_trail:
                 # replace stuff after start_trail with tree
@@ -474,8 +479,10 @@ class TreeBuilder(Observable):
                 self.invalidate_context(context)
 
             if offset:
+                if replace_pos_index < 0:
+                    self.replace_pos(context, len(context) + replace_pos_index, offset)
                 for p, i in context.ancestors_with_index():
-                    self.replace_pos(p, slice(i + 1, None), offset)
+                    self.replace_pos(p, i + 1, offset)
 
         return ReplaceResult(start, end + offset, lexicons)
 
@@ -497,14 +504,14 @@ class TreeBuilder(Observable):
         """
         self.root.lexicon = lexicon
 
-    def replace_pos(self, context, slice_, offset):
-        """Adjust the pos attribute of all tokens in the context's slice.
+    def replace_pos(self, context, index, offset):
+        """Adjust the pos attribute of all tokens in ``context[index:]``.
 
         This method is called by :meth:`replace_tree`.
         You can reimplement this method to notify others of the change.
 
         """
-        for t in tokens(context[slice_]):
+        for t in tokens(context[index:]):
             t.pos += offset
 
     def invalidate_context(self, context):
