@@ -182,25 +182,6 @@ class _Observer:
         return super().__gt__(other)
 
 
-class _EmitResult(list):
-    """Encapsulates the results of an Observer.emit() call.
-
-    When is it used as a context manager, it enters all results
-    that are a context manager as well.
-
-    """
-    __slots__ = ('exitstack')
-    def __enter__(self):
-        self.exitstack = s = contextlib.ExitStack()
-        for m in self:
-            if hasattr(m, '__enter__') and hasattr(m, '__exit__'):
-                s.enter_context(m)
-        return s.__enter__()
-
-    def __exit__(self, *exc):
-        return self.exitstack.__exit__(*exc)
-
-
 class Observable:
     """Simple base class for objects that need to announce events.
 
@@ -262,7 +243,7 @@ class Observable:
         the event. The ``priority`` determines the order the functions are
         called. Lower numbers are called first. If ``once`` is set to True, the
         function is called once and then removed from the list of callbacks. If
-        ``prepend_self`` is True, the callback is called with the treebuilder
+        ``prepend_self`` is True, the callback is called with the observable
         itself as first argument.
 
         """
@@ -322,21 +303,24 @@ class Observable:
     def emit(self, event, *args, **kwargs):
         """Call all callbacks for the event.
 
-        Returns a list of the return values of all callbacks. When using this
-        list in a ``with`` context, all return values that are context
-        managers, are entered. (The list is an :class:`_EmitResult` object that
-        extends Python's :class:`list` builtin, so that it can be used as a
-        context manager.)
+        Returns a :py:class:`contextlib.ExitStack` instance. When any of the
+        connected callbacks returns a context manager, that context is entered,
+        and added to the exit stack, so it is exited when the exit stack is
+        exited.
 
         """
-        results = _EmitResult()
+        s = contextlib.ExitStack()
         try:
             slots = self._callbacks[event]
         except KeyError:
-            return results
+            return s
         disconnect = []
         for i, observer in enumerate(slots):
-            results.append(observer.call(self, *args, **kwargs))
+            result = observer.call(self, *args, **kwargs)
+            try:
+                s.enter_context(result)
+            except Exception:
+                pass
             if observer.once:
                 disconnect.append(i)
         if disconnect:
@@ -344,7 +328,72 @@ class Observable:
                 del slots[i]
             if not slots:
                 del self._callbacks[event]
-        return results
+        return s
+
+
+class Switch:
+    """A context manager that evaluates to True when in a context, else to False.
+
+    Example::
+
+        clicking = Switch()
+
+        def myfunc():
+            with clicking:
+                blablabl()
+
+        # and elsewhere:
+        def blablabl():
+            if not clicking:
+                do_something()
+
+        # when blablabl() is called from myfunc, clicking evaluates to True,
+        # so do_something() is not called then.
+
+    A Switch can also be used in a class definition; via the descriptor
+    protocol it will then create per-instance Switch objects which will be
+    stored using a weak reference to the instance. For example::
+
+        class MyClass:
+            clicking = Switch()
+
+            def click_event(self, event):
+                with self.clicking:
+                    self.blablabla()
+
+            def blablabla(self):
+                do_something()
+                if not self.clicking:
+                    # this only runs when blablabla() was not called
+                    # from click_event()
+                    update_something()
+
+
+    """
+    __slots__ = ('_value',)
+
+    def __init__(self):
+        self._value = 0
+
+    def __enter__(self):
+        self._value += 1
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self._value -= 1
+
+    def __bool__(self):
+        return bool(self._value)
+
+    def __get__(self, instance, owner):
+        try:
+            return self._value[instance]
+        except TypeError:
+            # value still was 0, replace it with a weakref dict
+            self._value = weakref.WeakKeyDictionary()
+        except KeyError:
+            pass
+        s = self._value[instance] = type(self)()
+        return s
 
 
 class Symbol:
@@ -404,6 +453,34 @@ def cached_property(func):
     return property(cached_method(func))
 
 
+def fix_boundaries(stream, start, end):
+    """Yield all items from the stream of tuples.
+
+    The first two items of each tuple are regarded as pos and end. This
+    function adjusts the pos of the first item and the end of the last item so
+    that they do not stick out of the range start..end. If the pos of the first
+    item is below start, it is set to start; if the end of the last item is
+    beyond end, it is set to end.
+
+    If start == 0, the first item will never be adjusted; if end is None, the
+    last item will not be adjusted.
+
+    """
+    if start == 0 and end is None:
+        yield from stream   # do nothing
+    else:
+        stream = iter(stream)
+        for i in stream:
+            if i[0] < start:
+                i = type(i)((start, i[1], *i[2:]))
+            for j in stream:
+                yield i
+                i = j
+            if end is not None and i[1] > end:
+                i = type(i)((i[0], end, *i[2:]))
+            yield i
+
+
 def merge_adjacent(stream, factory=tuple):
     """Yield items from a stream of tuples.
 
@@ -473,13 +550,15 @@ def split_list(l, separator):
     Yields result lists that may be empty.
 
     """
-    try:
-        i = l.index(separator)
-    except ValueError:
-        yield l
-    else:
-        yield l[:i]
-        yield from split_list(l[i+1:], separator)
+    i = 0
+    while True:
+        try:
+            j = l.index(separator, i)
+        except ValueError:
+            yield l[i:]
+            return
+        yield l[i:j]
+        i = j + 1
 
 
 def quote(s):
